
#ifndef VTKPVVTKEXTENSIONSIOAMR_EXPORT_H
#define VTKPVVTKEXTENSIONSIOAMR_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSIOAMR_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSIOAMR_EXPORT
#  define VTKPVVTKEXTENSIONSIOAMR_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSIOAMR_EXPORT
#    ifdef VTKExtensionsIOAMR_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSIOAMR_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSIOAMR_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSIOAMR_NO_EXPORT
#    define VTKPVVTKEXTENSIONSIOAMR_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSIOAMR_DEPRECATED
#  define VTKPVVTKEXTENSIONSIOAMR_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSIOAMR_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSIOAMR_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSIOAMR_EXPORT VTKPVVTKEXTENSIONSIOAMR_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSIOAMR_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSIOAMR_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSIOAMR_NO_EXPORT VTKPVVTKEXTENSIONSIOAMR_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSIOAMR_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSIOAMR_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSIOAMR_EXPORT_H */
