
#ifndef VTKIOVISITBRIDGE_EXPORT_H
#define VTKIOVISITBRIDGE_EXPORT_H

#ifdef VTKIOVISITBRIDGE_STATIC_DEFINE
#  define VTKIOVISITBRIDGE_EXPORT
#  define VTKIOVISITBRIDGE_NO_EXPORT
#else
#  ifndef VTKIOVISITBRIDGE_EXPORT
#    ifdef IOVisItBridge_EXPORTS
        /* We are building this library */
#      define VTKIOVISITBRIDGE_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKIOVISITBRIDGE_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKIOVISITBRIDGE_NO_EXPORT
#    define VTKIOVISITBRIDGE_NO_EXPORT 
#  endif
#endif

#ifndef VTKIOVISITBRIDGE_DEPRECATED
#  define VTKIOVISITBRIDGE_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKIOVISITBRIDGE_DEPRECATED_EXPORT
#  define VTKIOVISITBRIDGE_DEPRECATED_EXPORT VTKIOVISITBRIDGE_EXPORT VTKIOVISITBRIDGE_DEPRECATED
#endif

#ifndef VTKIOVISITBRIDGE_DEPRECATED_NO_EXPORT
#  define VTKIOVISITBRIDGE_DEPRECATED_NO_EXPORT VTKIOVISITBRIDGE_NO_EXPORT VTKIOVISITBRIDGE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKIOVISITBRIDGE_NO_DEPRECATED
#    define VTKIOVISITBRIDGE_NO_DEPRECATED
#  endif
#endif

#endif /* VTKIOVISITBRIDGE_EXPORT_H */
